# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVELogger import *
from Ui_LogWindow import *

class OVELogWindow(QtGui.QDialog):
    LOAD_KEY = 'LogWindow/window'
    def __init__(self, app):
        QtGui.QDialog.__init__(self)
        self.app = app
        self.ui = Ui_LogWindow()
        self.ui.setupUi(self)
        if self.isLoadable():
            self.loadSettings()
        self.connect(OVELogger.Inst(), QtCore.SIGNAL("logUpdated()"), self.logUpdated)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("clicked(QAbstractButton *)"), self.xon_actionButton_Box_clicked)
        
    def xon_actionButton_Box_clicked(self, button):
        role = self.ui.buttonBox.buttonRole(button)
        if role == QtGui.QDialogButtonBox.ResetRole:
            OVELogger.Inst().reset()
            OVELog("Log reset")
        
    def logUpdated(self):
        self.ui.textBrowser.setText("\n".join(OVELogger.Inst().contents))
        self.ui.textBrowser.moveCursor(QtGui.QTextCursor.End)
        self.ui.textBrowser.ensureCursorVisible()

    def saveSettings(self):
        key = self.LOAD_KEY
        settings = QtCore.QSettings()
        settings.setValue(key+"/loadable", QVariant(True))
        settings.setValue(key+"/pos", QVariant(self.pos()))
        settings.setValue(key+"/size", QVariant(self.size()))
        settings.setValue(key+"/visible", QVariant(self.isVisible()))
    
    def loadSettings(self):
        key = self.LOAD_KEY
        settings = QtCore.QSettings()
        pos = settings.value(key+"/pos", QVariant(QtCore.QPoint(200, 200))).toPoint()
        size = settings.value(key+"/size", QVariant(QtCore.QSize(400, 400))).toSize()
        visible = settings.value(key+"/visible", QVariant(True)).toBool()
        self.resize(size)
        self.move(pos)
        self.setVisible(visible)

    @classmethod
    def isLoadable(cls):
        key = cls.LOAD_KEY
        settings = QtCore.QSettings()
        return settings.value(key+"/loadable", QVariant(False)).toBool()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'HostWindow.ui'
#
# Created: Fri May  7 17:20:33 2010
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

try:
    from OVEStandard import globalForcePySide
    if globalForcePySide: raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_HostWindow(object):
    def setupUi(self, HostWindow):
        HostWindow.setObjectName("HostWindow")
        HostWindow.setWindowModality(QtCore.Qt.WindowModal)
        HostWindow.resize(400,300)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HostWindow.sizePolicy().hasHeightForWidth())
        HostWindow.setSizePolicy(sizePolicy)
        self.gridLayout_2 = QtGui.QGridLayout(HostWindow)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtGui.QLabel(HostWindow)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label,0,0,1,1)
        self.hostAddressEdit = QtGui.QLineEdit(HostWindow)
        self.hostAddressEdit.setMinimumSize(QtCore.QSize(256,0))
        self.hostAddressEdit.setObjectName("hostAddressEdit")
        self.gridLayout.addWidget(self.hostAddressEdit,0,1,1,1)
        self.label_2 = QtGui.QLabel(HostWindow)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2,1,0,1,1)
        self.hostPasswordEdit = QtGui.QLineEdit(HostWindow)
        self.hostPasswordEdit.setMinimumSize(QtCore.QSize(256,0))
        self.hostPasswordEdit.setEchoMode(QtGui.QLineEdit.Password)
        self.hostPasswordEdit.setObjectName("hostPasswordEdit")
        self.gridLayout.addWidget(self.hostPasswordEdit,1,1,1,1)
        self.label_3 = QtGui.QLabel(HostWindow)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3,2,0,1,1)
        self.hostConnectTarget = QtGui.QLineEdit(HostWindow)
        self.hostConnectTarget.setMinimumSize(QtCore.QSize(256,0))
        self.hostConnectTarget.setObjectName("hostConnectTarget")
        self.gridLayout.addWidget(self.hostConnectTarget,2,1,1,1)
        self.gridLayout_2.addLayout(self.gridLayout,0,0,1,1)
        self.buttonBox = QtGui.QDialogButtonBox(HostWindow)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout_2.addWidget(self.buttonBox,1,0,1,1)
        self.label.setBuddy(self.hostAddressEdit)
        self.label_2.setBuddy(self.hostPasswordEdit)
        self.label_3.setBuddy(self.hostConnectTarget)

        self.retranslateUi(HostWindow)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("accepted()"),HostWindow.accept)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("rejected()"),HostWindow.reject)
        QtCore.QMetaObject.connectSlotsByName(HostWindow)
        HostWindow.setTabOrder(self.hostAddressEdit,self.hostPasswordEdit)
        HostWindow.setTabOrder(self.hostPasswordEdit,self.buttonBox)

    def retranslateUi(self, HostWindow):
        HostWindow.setWindowTitle(QtGui.QApplication.translate("HostWindow", "Host Properties", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("HostWindow", "Host name or IP", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("HostWindow", "SSH Password", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("HostWindow", "Connect target", None, QtGui.QApplication.UnicodeUTF8))




# Copyright (c) 2010 Citrix Systems, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from OVEStandard import *
from OVELogger import *
from Ui_HostWindow import *

class OVEHostWindow(QtGui.QDialog):
    DEFAULT_CONNECT_TARGET = 'unix:/var/run/openvswitch/db.sock'
    def __init__(self, parent, currentValues = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_HostWindow()
        self.ui.setupUi(self)
        self.resize(-1, -1)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("clicked(QAbstractButton *)"), self.xon_actionButton_Box_clicked)
        if currentValues is not None:
            self.ui.hostAddressEdit.setText(currentValues['address'])
            self.ui.hostPasswordEdit.setText(currentValues['password'])
            self.ui.hostConnectTarget.setText(currentValues.get('connectTarget', self.DEFAULT_CONNECT_TARGET))
            self.uuid = currentValues.get('uuid', str(uuid.uuid4()))
        else:
            self.ui.hostConnectTarget.setText(self.DEFAULT_CONNECT_TARGET)
            self.uuid = str(uuid.uuid4())
            self.accepted = None
    
    def xon_actionButton_Box_clicked(self, button):
        role = self.ui.buttonBox.buttonRole(button)
        if role == QtGui.QDialogButtonBox.AcceptRole:
            self.accepted = True
            self.close()
        elif role == QtGui.QDialogButtonBox.RejectRole:
            self.accepted = False
            self.close()

    def record(self):
        return {
            'accepted' : self.accepted,
            'uuid' : self.uuid,
            'address' : str(self.ui.hostAddressEdit.text()),
            'password' : str(self.ui.hostPasswordEdit.text()),
            'connectTarget' : str(self.ui.hostConnectTarget.text())
            }




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'LogWindow.ui'
#
# Created: Fri May  7 17:20:33 2010
#      by: PyQt4 UI code generator 4.4.2
#
# WARNING! All changes made in this file will be lost!

try:
    from OVEStandard import globalForcePySide
    if globalForcePySide: raise Exception()
    from PyQt4 import QtCore, QtGui
except:
    from PySide import QtCore, QtGui

class Ui_LogWindow(object):
    def setupUi(self, LogWindow):
        LogWindow.setObjectName("LogWindow")
        LogWindow.resize(735,558)
        self.gridLayout = QtGui.QGridLayout(LogWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.textBrowser = QtGui.QTextBrowser(LogWindow)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)
        self.buttonBox = QtGui.QDialogButtonBox(LogWindow)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close|QtGui.QDialogButtonBox.Reset)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.gridLayout.addLayout(self.verticalLayout,0,0,1,1)

        self.retranslateUi(LogWindow)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("accepted()"),LogWindow.accept)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("rejected()"),LogWindow.reject)
        QtCore.QMetaObject.connectSlotsByName(LogWindow)

    def retranslateUi(self, LogWindow):
        LogWindow.setWindowTitle(QtGui.QApplication.translate("LogWindow", "OVSDB Monitor Log", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/python
# Copyright (c) 2013 Nicira, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import optparse
import os
import shutil
import subprocess
import sys
import tempfile

ENV = os.environ
HOME = ENV["HOME"]
OVS_SRC = HOME + "/ovs"
ROOT = HOME + "/root"
PATH = "%(ovs)s/utilities:%(ovs)s/ovsdb:%(ovs)s/vswitchd" % {"ovs": OVS_SRC}

ENV["CFLAGS"] = "-g -O0"
ENV["PATH"] = PATH + ":" + ENV["PATH"]

options = None
parser = None
commands = []


def _sh(*args, **kwargs):
    print "------> " + " ".join(args)
    shell = len(args) == 1
    if kwargs.get("capture", False):
        proc = subprocess.Popen(args, stdout=subprocess.PIPE, shell=shell)
        return proc.stdout.readlines()
    elif kwargs.get("check", True):
        subprocess.check_call(args, shell=shell)
    else:
        subprocess.call(args, shell=shell)


def uname():
    return _sh("uname", "-r", capture=True)[0].strip()


def conf():
    tag()
    if options.clang:
        ENV["CC"] = "clang"

    configure = ["./configure", "--prefix=" + ROOT, "--localstatedir=" + ROOT,
                 "--with-logdir=%s/log" % ROOT, "--with-rundir=%s/run" % ROOT,
                 "--with-linux=/lib/modules/%s/build" % uname(),
                 "--with-dbdir=" + ROOT]

    if options.werror:
        configure.append("--enable-Werror")

    if options.cache_time:
        configure.append("--enable-cache-time")

    if options.mandir:
        configure.append("--mandir=" + options.mandir)

    _sh("./boot.sh")
    _sh(*configure)
commands.append(conf)


def make(args=""):
    make = "make -s -j 8 " + args
    try:
        _sh("cgcc", "--version", capture=True)
        # XXX: For some reason the clang build doesn't place nicely with
        # sparse.  At some point this needs to be figured out and this check
        # removed.
        if not options.clang:
            make += " C=1"
    except OSError:
        pass
    _sh(make)
commands.append(make)


def check():
    make("check")
commands.append(check)


def tag():
    ctags = ['ctags', '-R', '-f', '.tags']

    try:
        _sh(*(ctags + ['--exclude="datapath/"']))
    except:
        try:
            _sh(*ctags)  # Some versions of ctags don't have --exclude
        except:
            pass

    try:
        _sh('cscope', '-R', '-b')
    except:
        pass
commands.append(tag)


def kill():
    for proc in ["ovs-vswitchd", "ovsdb-server"]:
        if os.path.exists("%s/run/openvswitch/%s.pid" % (ROOT, proc)):
            _sh("ovs-appctl", "-t", proc, "exit", check=False)
            time.sleep(.1)
        _sh("sudo", "killall", "-q", "-2", proc, check=False)
commands.append(kill)


def reset():
    kill()
    if os.path.exists(ROOT):
        shutil.rmtree(ROOT)
    for dp in _sh("ovs-dpctl dump-dps", capture=True):
        _sh("ovs-dpctl", "del-dp", dp.strip())
commands.append(reset)


def run():
    kill()
    for d in ["log", "run"]:
        d = "%s/%s" % (ROOT, d)
        shutil.rmtree(d, ignore_errors=True)
        os.makedirs(d)

    pki_dir = ROOT + "/pki"
    if not os.path.exists(pki_dir):
        os.mkdir(pki_dir)
        os.chdir(pki_dir)
        _sh("ovs-pki init")
        _sh("ovs-pki req+sign ovsclient")
        os.chdir(OVS_SRC)

    if not os.path.exists(ROOT + "/conf.db"):
        _sh("ovsdb-tool", "create", ROOT + "/conf.db",
            OVS_SRC + "/vswitchd/vswitch.ovsschema")

    opts = ["--pidfile", "--log-file", "--enable-dummy"]

    _sh(*(["ovsdb-server",
           "--remote=punix:%s/run/db.sock" % ROOT,
           "--remote=db:Open_vSwitch,Open_vSwitch,manager_options",
           "--private-key=db:Open_vSwitch,SSL,private_key",
           "--certificate=db:Open_vSwitch,SSL,certificate",
           "--bootstrap-ca-cert=db:Open_vSwitch,SSL,ca_cert",
           "--detach", "-vconsole:off"] + opts))

    _sh("ovs-vsctl --no-wait --bootstrap set-ssl %s/ovsclient-privkey.pem" \
        " %s/ovsclient-cert.pem %s/vswitchd.cacert"
        % (pki_dir, pki_dir, pki_dir))
    version = _sh("ovs-vsctl --no-wait --version", capture=True)
    version = version[0].strip().split()[3]
    root_uuid = _sh("ovs-vsctl --no-wait --bare list Open_vSwitch",
                    capture=True)[0].strip()
    _sh("ovs-vsctl --no-wait set Open_vSwitch %s ovs_version=%s"
        % (root_uuid, version))

    cmd = [OVS_SRC + "/vswitchd/ovs-vswitchd"]
    if options.gdb:
        cmd = ["gdb", "--args"] + cmd
    elif options.valgrind:
        cmd = ["valgrind", "--track-origins=yes",
               "--suppressions=%s/tests/glibc.supp" % OVS_SRC,
               "--suppressions=%s/tests/openssl.supp" % OVS_SRC] + cmd
    else:
        cmd = ["sudo"] + cmd
        opts = opts + ["-vconsole:off", "--detach"]
    _sh(*(cmd + opts))
commands.append(run)


def modinst():
    if not os.path.exists("/lib/modules"):
        print "Missing modules directory.  Is this a Linux system?"
        sys.exit(1)

    try:
        _sh("rmmod", "openvswitch")
    except subprocess.CalledProcessError, e:
        pass  # Module isn't loaded

    try:
        _sh("rm /lib/modules/%s/extra/openvswitch.ko" % uname())
    except subprocess.CalledProcessError, e:
        pass  # Module isn't installed

    conf()
    make()
    make("modules_install")

    _sh("modprobe", "openvswitch")
    _sh("dmesg | grep openvswitch | tail -1")
commands.append(modinst)


def env():
    print "export PATH=" + ENV["PATH"]
commands.append(env)


def doc():
    parser.print_help()
    print \
"""
This program is designed to help developers build and run Open vSwitch without
necessarily needing to know the gory details. Given some basic requirements
(described below), it can be used to build and run Open vSwitch, keeping
runtime files in the user's home directory.

Basic Configuration:
    # This section can be run as a script on ubuntu systems.

    # First install the basic requirements needed to build Open vSwitch.
    sudo apt-get install git build-essential libtool autoconf pkg-config \\
            libssl-dev pkg-config gdb linux-headers-`uname -r`

    # Next clone the Open vSwitch source.
    git clone git://git.openvswitch.org/openvswitch %(ovs)s

    # Setup environment variables.
    `%(v)s env`

    # Build the switch.
    %(v)s conf make

    # Install the kernel module
    sudo insmod %(ovs)s/datapath/linux/openvswitch.ko

    # Run the switch.
    %(v)s run

Commands:
    conf    - Configure the ovs source.
    make    - Build the source (must have been configured).
    check   - Run the unit tests.
    tag     - Run ctags and cscope over the source.
    kill    - Kill all running instances of ovs.
    reset   - Reset any runtime configuration in %(run)s.
    run     - Run ovs.
    modinst - Build ovs and install the kernel module.
    env     - Print the required path environment variable.
    doc     - Print this message.
""" % {"ovs": OVS_SRC, "v": sys.argv[0], "run": ROOT}
    sys.exit(0)
commands.append(doc)


def main():
    global options
    global parser

    description = "Open vSwitch developer configuration. Try `%prog doc`."
    cmd_names = [c.__name__ for c in commands]
    parser = optparse.OptionParser(usage="usage: %prog"
                                   + " [options] [%s] ..."
                                   % "|".join(cmd_names),
                                   description=description)

    group = optparse.OptionGroup(parser, "conf")
    group.add_option("--disable-Werror", dest="werror", action="store_false",
                     default=True, help="compile without the Werror flag")
    group.add_option("--cache-time", dest="cache_time",
                     action="store_true", help="configure with cached timing")
    group.add_option("--mandir", dest="mandir", metavar="MANDIR",
                     help="configure the man documentation install directory")
    parser.add_option_group(group)

    group = optparse.OptionGroup(parser, "run")
    group.add_option("-g", "--gdb", dest="gdb", action="store_true",
                     help="run ovs-vswitchd under gdb")
    group.add_option("--valgrind", dest="valgrind", action="store_true",
                     help="run ovs-vswitchd under valgrind")
    group.add_option("--clang", dest="clang", action="store_true",
                     help="build ovs-vswitchd with clang")
    parser.add_option_group(group)

    options, args = parser.parse_args()

    for arg in args:
        if arg not in cmd_names:
            print "Unknown argument " + arg
            doc()

    try:
        os.chdir(OVS_SRC)
    except OSError:
        print "Missing %s." % OVS_SRC
        doc()

    for arg in args:
        for cmd in commands:
            if arg == cmd.__name__:
                cmd()


if __name__ == '__main__':
    main()
